# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# This controller handles things that Admins do, like create and modify Users.
class AdminController < ApplicationController

  before_filter :authorize
  before_filter :authorize2
  


  def index
    redirect_to :action => 'list'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def list
    @user_pages, @users = paginate :users, :per_page => 10
    render :layout => 'simple'
  end

  def show
    @user = User.find(params[:id])
    render :layout => 'simple'
  end

  def new
    @user = User.new
  end

  def create
    @user = User.new(params[:user])
    if params[:user][:user_level].to_i >= session[:level] && session[:level] < 3
      flash[:notice] = "You may only create users of lesser level than you."
      render :action => 'new'
    else
      if @user.save
        flash[:notice] = 'User was successfully created.'
        redirect_to :action => 'list'
      else
        render :action => 'new'
      end
    end
  end  
  def edit
    @user = User.find(params[:id])
    if @user.user_level >= session[:level] && session[:level] < 3
      flash[:notice] = "You are not authorized to edit that user."
      redirect_to :action => 'list'
    else
      render :layout => 'simple'
    end
  end

  def update
    @user = User.find(params[:id])
    if params[:user][:user_level].to_i >= session[:level] && session[:level] < 3
      flash[:notice] = "You may only create users of lesser level than you."
      render :action => 'edit'
    else
      if @user.update_attributes(params[:user])
        flash[:notice] = 'User was successfully updated.'
        redirect_to :action => 'show', :id => @user
      else
        render :action => 'edit'
      end
    end
  end

  def destroy
    @user_to_delete = User.find(params[:id])
    if @user_to_delete.user_level >= session[:level]
      flash[:notice] = "You may only delete users of lesser level than you."
      redirect_to :action => 'list'
    else
      @user_to_delete.destroy
      redirect_to :action => 'list'
    end
  end
  
  def undo
    @change_pages, @changes = paginate :changes, :per_page => 10, :order => 'date DESC'
    if request.post?	  
      @s = @d = ""
     # @changes.each { |c| @s << c.id.to_s + " " if params[:post][c.id.to_s] == '1' }
     # @changes.each { |c| @d << c.id.to_s + " " }
     # raise @d
      params[:post].each { |x| Change.find(x[0].to_i).undo(session[:user_id]) if x[1] == '1' && Change.find(:first, :conditions => {:id => x[0].to_i} ) }
      redirect_to :action => 'undo'
    else
      render :layout => 'simple'
    end  
  end
end
