# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

#This controller governs all actions pertaining to the browser portion of the application.
class MetarepController < ApplicationController
caches_page :tree
model :user
model :column
model :tree_location

before_filter :check_flag
before_filter :check_tree_locations_table

#If the user is flagged, this will redirect him to change his password.
def check_flag
  if session[:user_id]
    if User.find(session[:user_id]).flag == 'RESET'
      flash[:notice] = "Your password is a temporary password. Please change your password."
      redirect_to :action => 'change_password', :controller => 'authorize'
    end
  end
end

#Make sure the tree locations table is not empty
def check_tree_locations_table
  if TreeLocation.count == 0
    expire_tree_cache
    flash[:notice] = "The metadata browser's search index had to be refreshed.  Sorry about the wait!"
    redirect_to :action => 'browser'
  end
end

#Renders a blank page in the details iframe to start.
  def default
    session[:contextobject] = nil
    session[:contextid] = nil
    render :layout => 'simple'
  end

#Directs users to the correct action upon initial visit.
  def index
    redirect_to :action => 'browser'
  end

#Displays information about the selected database in the details iframe.
  def showDatabase
    @database = Database.find(params[:id])
    session[:contextobject] = 'Database'
    session[:contextid] = @database.id
    render :layout => 'simple'    
  end

#Displays information about the selected schema in the details iframe.
  def showSchema
    @schema = Schema.find(params[:id])
    session[:contextobject] = 'Schema'
    session[:contextid] = @schema.id
    render :layout => 'simple'    
  end

#Displays information about the selected table in the details iframe.
  def showTable
    @table = Table.find(params[:id])
    session[:contextobject] = 'Table'
    session[:contextid] = @table.id
    render :layout => 'simple'    
  end

#Displays information about the selected column in the details iframe.
  def showColumn
    @column = Column.find(params[:id])
    session[:contextobject] = 'Column'
    session[:contextid] = @column.id
    render :layout => 'simple'
  end
  
  def showReport
    @report = Report.find(params[:id])
    session[:contextobject] = 'Report'
    session[:contextid] = @report.id
    render :layout => 'simple'
  end
  
  def showReportFolder
    @report_folder = ReportFolder.find(params[:id])
    session[:contextobject] = 'ReportFolder'
    session[:contextid] = @report_folder.id
    render :layout => 'simple'
  end

#Displays the browser layout. Tells the user he's logged in if he is logged in.
  def browser
    flash.keep
    @user_name = User.find(session[:user_id]).email if session[:user_id]
    render :layout => 'metarep'    
  end

#Displays the tree of objects.
  def tree
    @doTree = false
    if SchemaType.count < 1
      flash[:error] = "Sorry, you have not added any entries to the metadata.schema_types table.  There must be at least one schema_type, and each schema must have a schema_type_id for it to be visible in the Metadata browser. "
    else
      @database = Database.find_by_database_nm(Database.find_first.currentDatabaseName)
      @schema_types = SchemaType.find(:all, :order => "display_seq")
      @schemas = Schema.find (:all, :conditions => "database_id = #{@database.id}", :order => "schema_nm")
      @doTree = true
    end
    render :layout => 'tree' 
  end

#Saves a database's description after a user edit.
  def update_database_description
    if session[:user_id]
      @database = Database.find(params[:id])
      @change = Change.new({:object_path => @database.database_nm, :data_changed => "database_desc", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @database.database_desc = params[:value]
      @database.save
      redirect_to :action => 'showDatabase', :id => params[:id]
    end
  end

#Saves a schema's description after a user edit.
  def update_schema_description
   if session[:user_id]
      @schema = Schema.find(params[:id])
      @change = Change.new({:object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => "schema_desc", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @schema.schema_desc = params[:value]
      @schema.save
      redirect_to :action => 'showSchema', :id => params[:id]
    end
  end

#Saves a schema's title after a user edit.
  def update_schema_title
   if session[:user_id]
      @schema = Schema.find(params[:id])
      @change = Change.new({:object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => "schema_title", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @schema.schema_title = params[:value]
      @schema.save
      redirect_to :action => 'showSchema', :id => params[:id]
    end
  end
 
#Saves a table's description after a user edit. 
  def update_table_description
    if session[:user_id]
      @table = Table.find(params[:id])
      @change = Change.new({:object_path => @table.Schema.Database.database_nm + '/' + @table.Schema.schema_nm + '/' + @table.table_nm, :data_changed => "table_desc", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @table.table_desc = params[:value]
      @table.save
      redirect_to :action => 'showTable', :id => params[:id]
    end
  end

#Saves a column's description after a user edit.
  def update_column_description
   if session[:user_id]
     @column = Column.find(params[:id]) 
     @change = Change.new({:object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.table_nm + '/' + @column.column_nm, :data_changed => "column_desc", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @column.column_desc = params[:value]
      @column.save
      redirect_to :action => 'showColumn', :id => params[:id]
    end
  end
  
#Saves a column's sample data after a user edit.  
  def update_column_sample_data_txt
   if session[:user_id]
      @column = Column.find(params[:id])   
      @change = Change.new({:object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.table_nm + '/' + @column.column_nm, :data_changed => "sample_data_txt", 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @column.sample_data_txt = params[:value]
      @column.save
      redirect_to :action => 'showColumn', :id => params[:id]
    end 
  end

#Saves a report's attribute, given by the field parameter, after a user edit.
#The field is specified to avoid having to create too many functions, as there are
#many editable attributes in a report.
  def update_report
    if session[:user_id]
      @report = Report.find(params[:id])   
      @change = Change.new({:object_path => @report.path, :data_changed => params[:field], 
      :user_id => session[:user_id],                
      :new_value => params[:value], :date => Time.now})                                                                                                       
      @change.save
      @report.update_attribute(params[:field], params[:value])
      redirect_to :action => 'showReport', :id => params[:id]
    end  
  end

#Shows the history of changes to this database object's description.
  def showDatabaseHistory
    @database = Database.find(params[:id])
    @description_changes = Change.find(:all, :conditions => { :object_path => @database.database_nm }, :order => "date desc" )
    render :layout => 'simple'
  end

#Shows the history of changes to this schema object's description and title.
  def showSchemaHistory
    @schema = Schema.find(params[:id])
    @description_changes = Change.find(:all, :conditions => { :object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => 'schema_desc' }, :order => "date desc" )
    @title_changes = Change.find(:all, :conditions => { :object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => 'schema_title' }, :order => "date desc" )
    render :layout => 'simple'
  end

#Shows the history of changes to this table object's description.
  def showTableHistory
    @table = Table.find(params[:id])
    @description_changes = Change.find(:all, :conditions => { :object_path => @table.Schema.Database.database_nm + '/' + @table.Schema.schema_nm + '/' + @table.table_nm }, :order => "date desc" )
    render :layout => 'simple'
  end
 
#Shows the history of changes to this column object's description and sample data. 
  def showColumnHistory
    @column = Column.find(params[:id])
    @sample_data_changes = Change.find(:all, :conditions => { :object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.table_nm + '/' + @column.column_nm, :data_changed => 'sample_data_txt' }, :order => "date desc" )
    @description_changes = Change.find(:all, :conditions => { :object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.table_nm + '/' + @column.column_nm, :data_changed => 'column_desc' }, :order => "date desc" )
    render :layout => 'simple'
  end

#Shows the history of changes to this report object's description and sample data.
  def showReportHistory
    @report = Report.find(params[:id])
    @author_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'author_nm' }, :order => "date desc" )
    @url_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'report_url' }, :order => "date desc" )
    @source_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'source_nm' }, :order => "date desc" )
    @purpose_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'purpose_txt' }, :order => "date desc" )
    @audience_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'audience_txt' }, :order => "date desc" )
    @selection_criteria_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'selctn_criteria_txt' }, :order => "date desc" )   
    @business_owner_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'bus_owner_nm' }, :order => "date desc" )  
    @report_spnsr_changes = Change.find(:all, :conditions => { :object_path => @report.path, :data_changed => 'report_spnsr_nm' }, :order => "date desc" )      
    @description_changes = Change.find(:all, :conditions => {:object_path => @report.path, :data_changed => 'dsc_txt' }, :order => "date desc" )
    render :layout => 'simple'
  end
  
#Expires the tree cache. This will rebuild the tree location mappings
#needed to link tree nodes with object id's and force a rebuilding of
#the tree on the next visit. A script visits this action on a regular basis.
  def expire_tree_cache
    #run this after the metadata is reloaded
    expire_page(:action => 'tree')
    tree_location_build
  end
  
#Searches all objects in the given context for the given search term. Reports all results, paginating
#with 10 per page.
  def search
    @databaseName = Database.find_first.currentDatabaseName
    #This function will create arrays containing all records of schemas, columns, and tables that
    # satisfy the LIKE condition with the search term.
    if request.post?
    session[:search_term] = params[:post][:search_term].strip               #Get the search term
    if session[:contextobject] == 'Column' or !session[:contextobject]
      session[:contextobject] = 'Database'
      session[:contextid] = Database.find_by_database_nm(@databaseName).id
    end
    if session[:contextobject] == "Report" or session[:contextobject] == "ReportFolder"
    session[:contextobject] = "ReportFolder"
    session[:contextid] = ReportFolder.find_by_report_folder_nm('Reports').id
    end
    @search = Search.new(:search_term => session[:search_term], 
      :context_object_type => session[:contextobject], :context_object_id => session[:contextid])
    @search.search_date = Time.now
    @search.user_id = session[:user_id]
    @search.save
    end
    add_to_sortable_columns('listing', :model => DwObject, :field => 'objectTitle', :alias => 'objectTitle') 
    add_to_sortable_columns('listing', :model => DwObject, :field => 'objectType', :alias => 'objectType') 
    add_to_sortable_columns('listing', :model => DwObject, :field => 'path', :alias => 'path')
    
    #This is the part that does the searchy magic
    unless session[:contextobject] == "Report" or session[:contextobject] == "ReportFolder"
    @dw_object_pages, @dw_objects = paginate :dw_objects, :per_page => 10, :order => sortable_order('listing', 
       :model => DwObject, 
       :field => 'path',
       :sort_direction => :asc),
	 :conditions => "((cast(objectDesc as varchar(1000)) like 
    '%" + session[:search_term] + "%' or cast(objectTitle as varchar(1000)) like '%" + 
      session[:search_term] + "%')) and path is not null" + context_condition(session[:contextobject], session[:contextid]).to_s
    else
    @dw_object_pages, @dw_objects = paginate :dw_objects, :per_page => 10, :order => sortable_order('listing', 
       :model => DwObject, 
       :field => 'objectTitle',
       :sort_direction => :asc),
	 :conditions => "objectType = 'Report' and (cast(objectDesc as varchar(1000)) like 
    '%" + session[:search_term] + "%' or cast(objectTitle as varchar(1000)) like '%" + 
      session[:search_term] + "%')"
     @dw_objects.each { |r| r.path = Report.find(r.dw_object_id).path }
    end
    @count = @dw_object_pages.item_count
  render :layout => 'simple'
  end

#Call this directly if new schemas, tables, or columns are added. This function rebuilds linkages to the tree.
  def tree_location_build
    TreeLocation.destroy_all
    @database = Database.find_by_database_nm(Database.find_first.currentDatabaseName)
    @schema_types = SchemaType.find(:all, :order => "display_seq")     
    @tree_loc = TreeLocation.new({:path => '/metarep/showDatabase/' + 
      @database.id.to_s, :tree_id => 0})
    @tree_loc.save
    @count = 0
    
    for schema_type in @schema_types
      @count += 1
      @tree_loc = TreeLocation.new({:path => '/metarep/showSchemaType/' + 
            schema_type.id.to_s, :tree_id => @count})
      @tree_loc.save
    
      @schemas = Schema.find(:all, :conditions => { :schema_type_id => schema_type.id, :database_id => @database.id}, :order => "schema_nm") 
        for schema in @schemas
          @count += 1
          @tree_loc = TreeLocation.new({:path => '/metarep/showSchema/' + 
            schema.id.to_s, :tree_id => @count})
          @tree_loc.save
          for table in schema.tables
            @count += 1
                  @tree_loc = TreeLocation.new({:path => '/metarep/showTable/' + 
              table.id.to_s, :tree_id => @count})
            @tree_loc.save
              for column in table.columns
                @count += 1
                      @tree_loc = TreeLocation.new({:path => '/metarep/showColumn/'+ 
                  column.id.to_s, :tree_id => @count})
                      @tree_loc.save
              end    
          end
        end
    end
    
    unless ReportFolder.find_by_report_folder_nm('Reports').nil?
      f = ReportFolder.find_by_report_folder_nm('Reports')      
      # The root node of Northwestern's reports server is called "Reports".  Please this with yours.
      @tree_loc = TreeLocation.new({:path => '/metarep/showReportFolder/'+ 
      f.id.to_s, :tree_id => @count+1})
      @tree_loc.save
      # After adding the root node, build the rest of the Report tree
      report_tree_location_build (@count, f) #Build the report part of the tree
    end
  end
  
  #Builds the report part of the tree. Only to be called from tree_location_build
  def report_tree_location_build (start, folder)
    @count = start
    for f in folder.children
      @count += 1
      @count = report_tree_location_build (@count, f)
    end
    for r in folder.reports
      @count += 1
      @tree_loc = TreeLocation.new({:path => '/metarep/showReport/'+ 
	r.id.to_s, :tree_id => @count})
      @tree_loc.save
    end
    return @count    
  end

  #Call this directly to store all information in the EDW in the change history. They will be commented with FULL STORE.
  #The use of this is to revert to the last FULL STORE in case of a major site/database problem destroying all the site data.
  #Individual reverts will also be possible; this is just to save time.
  #Only a SuperAdmin or greater will succeed in calling this.
  def store_all_info
  if session[:user_id] and session[:level] > 2
    @database = Database.find_by_database_nm(Database.find_first.currentDatabaseName)
    @change = Change.new({:object_path => @database.database_nm, :data_changed => "database_desc", 
      :user_id => session[:user_id], :comment => 'FULL STORE',    #FULL STORE comment means the change was generated by this command.
      :new_value => @database.description, :date => Time.now}) 
    @change.save
    for schema in @database.schemas
      @change = Change.new({:object_path => schema.Database.database_nm + '/' + schema.schema_nm, :data_changed => "schema_desc", 
      :user_id => session[:user_id], :comment => 'FULL STORE',               
      :new_value => schema.description, :date => Time.now})
      @change.save
      @change = Change.new({:object_path => schema.Database.database_nm + '/' + schema.schema_nm, :data_changed => "schema_title", 
      :user_id => session[:user_id], :comment => 'FULL STORE',               
      :new_value => schema.title, :date => Time.now})
      @change.save
      for table in schema.tables
        @change = Change.new({:object_path => table.Schema.Database.database_nm + '/' + table.Schema.schema_nm + '/' + table.table_nm, :data_changed => "description", 
        :user_id => session[:user_id], :comment => 'FULL STORE',             
        :new_value => table.description, :date => Time.now})                                                                                                       
        @change.save
	for column in table.columns
          @change = Change.new({:object_path => column.Table.Schema.Database.database_nm + '/' + column.Table.Schema.schema_nm + '/' + column.Table.table_nm + '/' + column.column_nm, :data_changed => "column_desc", 
          :user_id => session[:user_id], :comment => 'FULL STORE',                
          :new_value => column.description, :date => Time.now})                                                                                                       
          @change.save
          @change = Change.new({:object_path => column.Table.Schema.Database.database_nm + '/' + column.Table.Schema.schema_nm + '/' + column.Table.table_nm + '/' + column.column_nm, :data_changed => "sample_data_txt", 
          :user_id => session[:user_id], :comment => 'FULL STORE',                
          :new_value => column.sample_data_txt, :date => Time.now})                                                                                                       
          @change.save
	end    
      end
    end
  end
  redirect_to :action => 'browser' #Goto the metadata browser when done.
  end
  
#If this is a GET request, displays the editor.
#If this is a POST request, saves the user's edit to the database.
  def edit
    @object_type = session[:contextobject]
    @dw_object_id = session[:contextid]
    if request.get?
      session[:field] = @field = params[:field]    
      @human_field = @field
      @human_field = "purpose" if @field == "purpose_txt"
      @human_field = "source" if @field == "source_nm"
      @human_field = "selection criteria" if @field == "selctn_criteria_txt"
      @human_field = "author" if @field == "author_nm"
      @human_field = "business owner" if @field == "bus_owner_nm"
      @human_field = "url" if @field == "report_url"
      @human_field = "sample data" if @field == "sample_data_txt"
      @human_field = "sponsor" if @field == "report_spnsr_nm"
      @human_field = "description" if @field == "dsc_txt"
      case @object_type
      when 'Database'
        @object_name = Database.find(@dw_object_id).database_nm
	@DwObject = Database.find(@dw_object_id)
      when 'Schema'
        @object_name = Schema.find(@dw_object_id).schema_nm
	@DwObject = Schema.find(@dw_object_id)
      when 'Report'
        @object_name = Report.find(@dw_object_id).report_nm
	@DwObject = Report.find(@dw_object_id)
      when 'Table'
        @object_name = Table.find(@dw_object_id).table_nm
	@DwObject = Table.find(@dw_object_id)
      when 'Column'
	@object_name = Column.find(@dw_object_id).column_nm
	@DwObject = Column.find(@dw_object_id)
      end
      render :layout => 'simple'
    else
      @field = session[:field]
      @new_value = params[:post][:new_value]
      case @object_type
      when 'Database'
        redirect_to :action => 'update_database_description', :value => @new_value, :id => @dw_object_id
      when 'Schema'
        redirect_to :action => 'update_schema_title', :value => @new_value, :id => @dw_object_id if @field == 'schema_title'
	redirect_to :action => 'update_schema_description', :value => @new_value, :id => @dw_object_id if @field == 'schema_desc'
      when 'Table'
        redirect_to :action => 'update_table_description', :value => @new_value, :id => @dw_object_id
      when 'Column'
        redirect_to :action => 'update_column_sample_data_txt', :value => @new_value, :id => @dw_object_id if @field == 'sample_data_txt'
        redirect_to :action => 'update_column_description', :value => @new_value, :id => @dw_object_id if @field == 'column_desc'
      when 'Report'
        redirect_to :action => 'update_report', :value => @new_value, :field => @field, :id => @dw_object_id
      end
    end
  end
  
  #Reverts a description to the value stored in the selected change.
  def change_revert
    @change = Change.find(params[:id])
    @change.revert(session[:user_id])
    redirect_to :back
  end
  
  #Revert the element specified in this change to the version before this change's version. That is, perform an "undo" on this change.
  #Unlike REVERT, this action DELETES changes until the desired version is reached.
  def undo_change
    @change = Change.find(params[:id])
    @change.undo(session[:user_id])
    redirect_to :back 
  end
  
  #Shows information about either Subject Areas or Source Data in general
  def showSchemaType
    @schema_type = SchemaType.find(params[:id])    
    session[:contextobject] = 'SchemaType'
    session[:contextid] = @schema_type.id  
    render :layout => 'simple'    
  end
  
  def showNoReports
    render :layout => 'simple'    
  end
end
