# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# Methods added to this helper will be available to all templates in the application.
module ApplicationHelper

  def section_link(name,options)
      if options[:action] == @current_action and options[:controller] == @current_controller
         link_to(name, options, :class => 'active')
      else
        link_to(name,options)
      end
    end


   def editable_content(options)
     options[:content] = { :element => 'span' }.merge(options[:content])
     options[:url] = {}.merge(options[:url])
     options[:ajax] = { :okText => "'Save'", :cancelText => "'Cancel'"}.merge(options[:ajax] || {})
     script = Array.new
     script << "new Ajax.InPlaceEditor("
     script << "  '#{options[:content][:options][:id]}',"
     script << "  '#{url_for(options[:url])}',"
     script << "  {"
     script << options[:ajax].map{ |key, value| "#{key.to_s}: #{value}" }.join(", ")
     script << "  }"
     script << ")"

     content_tag(
       options[:content][:element],
       options[:content][:text],
       options[:content][:options]
     ) + javascript_tag( script.join("\n") )
   end
   
     #Similar to h, but this method will also automatically create links to replace any text between [[ and ]].
   def h2(text)
    text = h text
    text.gsub(/\[\[[^\]\[]+\]\]/) {|s|  try_to_link(s)}
   end
  
  #Attempts to create a link from the given text starting with [[ and ending in ]].
  #If a suitable object to link to cannot be found, returns plain text.
  #Otherwise, calls generate_link to create a link to the DwObject.
   def try_to_link(text)
      text = text.slice(2..text.length-3)
      if text =~ /^http:.+/ 
        return generate_url_link (text)
      end
      if text =~ /\A([^@\s]+)@(nmh.org|nmff.org|northwestern.edu)\Z/i
        return generate_mailto_link (text)
      end
      @dw_object = DwObject.find_by_path(text)
      if !@dw_object
           split_text = text.split(/[^a-zA-Z0-9\-_]/)
           case split_text.length
             when 3
               @dw_object = DwObject.find_by_path('EDW/'+text)
             when 2
               @dw_object = DwObject.find_by_path('EDW/'+text)
               if session[:contextobject] == 'SchemaType' && !@dw_object
                 theSchemaType = SchemaType.find(session[:contextid])
                 path_text = 'EDW/' + theSchemaType.schema_type_nm
                       @dw_object = DwObject.find_by_path(path_text + '/' + text)               
               elsif session[:contextobject] == 'Schema' && !@dw_object
                 theSchema = Schema.find(session[:contextid])
                 path_text = 'EDW/' + theSchema.schema_nm
                       @dw_object = DwObject.find_by_path(path_text + '/' + text)
               elsif session[:contextobject] == 'Table' && !@dw_object
                 theTable = Table.find(session[:contextid])
                 path_text = 'EDW/' + theTable.Schema.schema_nm
                 @dw_object = DwObject.find_by_path(path_text + '/' + text)
               end
             when 1
               @dw_object = DwObject.find_by_path('EDW/'+text)
               if session[:contextobject] == 'SchemaType' && !@dw_object
                 theSchema = Schema.find(session[:contextid])
                 path_text = 'EDW/' + theSchemaType.schema_type_nm
                       @dw_object = DwObject.find_by_path(path_text + '/' + text)               
               elsif session[:contextobject] == 'Schema' && !@dw_object
                 theSchema = Schema.find(session[:contextid])
                 path_text = 'EDW/' + theSchema.schema_nm
                       @dw_object = DwObject.find_by_path(path_text + '/' + text)
               elsif session[:contextobject] == 'Table' && !@dw_object
                 theTable = Table.find(session[:contextid])
                 path_text = 'EDW/' + theTable.Schema.schema_nm
                 @dw_object = DwObject.find_by_path(path_text + '/' + text)
                 @dw_object = DwObject.find_by_path(path_text + '/' + theTable.table_nm + '/' + text) if !@dw_object
               elsif !@dw_object && session[:contextobject] == 'Column'
                 theColumn = Column.find(session[:contextid])
                 path_text = 'EDW/' + theColumn.Table.Schema.schema_nm + '/' + theColumn.Table.table_nm
                 @dw_object = DwObject.find_by_path(path_text + '/' + text)
                 @dw_object = DwObject.find_by_path(path_text) if !@dw_object
             end
           end
        end
      return generate_object_link (@dw_object, text) if @dw_object
      else return text
   end
   
   #Returns HTML to create a link to the DwObject.
   def generate_object_link (dw_object, text)
	object_type = ''
	if dw_object.objectType == 'Table' 
		 object_type = '_table'
	elsif dw_object.objectType == 'Column' 
		object_type = '_column' 
	end
	return "<a href = '" + dw_object.link_path + "' onClick = 'parent.tree.theTree.a_index[#{dw_object.tree_id}].select#{object_type}()'>#{text}</a>" 
   end

   def generate_url_link (text)
	text = text.split('|')
        return "<a href = '#{text[0]}' target = 'new'>#{text[1]||text[0]}</a>"
   end
    
   def generate_mailto_link (text)
	return generate_url_link ("mailto:#{text}|#{text}")
   end
end
