# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# This model represents the changes table.  This table records changes to metadata in the repository.
class Change < ActiveRecord::Base
  belongs_to :User
  
  #Changes the object and data field represented in this change object to the value specified in the change object.
  def revert (author)
    @dw_object = DwObject.find_by_path(self.object_path)

    if @dw_object
    case @dw_object.objectType
    when 'Database'
      @database = Database.find(@dw_object.dw_object_id)
      @database.database_desc = self.new_value if self.data_changed == 'database_desc'
      @database.save
      @new_change = Change.new({:object_path => @database.database_nm, :data_changed => "database_desc", 
        :user_id => author, :comment => 'REVERT',             
        :new_value => self.new_value, :date => Time.now})                                                                                                       
      @new_change.save
    when 'Schema'
      @schema = Schema.find(@dw_object.dw_object_id)
      @schema.schema_desc = self.new_value if self.data_changed == 'schema_desc'
      @schema.schema_title = self.new_value if self.data_changed == 'schema_title'
      @schema.save
      @new_change = Change.new({:object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => "schema_desc", 
        :user_id => author, :comment => 'REVERT',             
        :new_value => self.new_value, :date => Time.now}) if self.data_changed == 'schema_desc' 
      @new_change = Change.new({:object_path => @schema.Database.database_nm + '/' + @schema.schema_nm, :data_changed => "schema_title", 
        :user_id => author, :comment => 'REVERT',             
        :new_value => self.new_value, :date => Time.now}) if self.data_changed == 'schema_title'     
      @new_change.save
    when 'Table'
      @table = Table.find(@dw_object.dw_object_id)
      @table.table_desc = self.new_value if self.data_changed == 'table_desc'
      @table.save
      @new_change = Change.new({:object_path => @table.Schema.Database.database_nm + '/' + @table.Schema.schema_nm + '/' + @table.table_nm, :data_changed => "table_desc", 
        :user_id => author, :comment => 'REVERT',             
        :new_value => self.new_value, :date => Time.now})                                                                                                       
      @new_change.save
    when 'Column'
      @column = Column.find(@dw_object.dw_object_id)
      @column.column_desc = self.new_value if self.data_changed == 'column_desc'
      @column.sample_data_txt = self.new_value if self.data_changed == 'sample_data_txt'
      @column.save
      @new_change = Change.new({:object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.schema_nm + '/' + @column.column_nm, :data_changed => "sample_data_txt", 
         :user_id => author, :comment => 'REVERT',                
         :new_value => self.new_value, :date => Time.now}) if self.data_changed == 'sample_data_txt'                                                                                                
      @new_change = Change.new({:object_path => @column.Table.Schema.Database.database_nm + '/' + @column.Table.Schema.schema_nm + '/' + @column.Table.table_nm + '/' + @column.column_nm, :data_changed => "column_desc", 
         :user_id => author, :comment => 'REVERT',                
         :new_value => self.new_value, :date => Time.now}) if self.data_changed == 'column_desc'                
      @new_change.save
     end
     else
     @reports = Report.find_all()
     for report in @reports
       if report.path == self.object_path
	 @the_report = report
       end
     end
     @the_report[self.data_changed] = self.new_value
     @the_report.save
     @new_change = Change.new({:object_path => @the_report.path, :data_changed => self.data_changed,
       :user_id => author, :comment => 'REVERT',
       :new_value => self.new_value, :date => Time.now})
     @new_change.save
     end
   end
   
   #Restores the status of the object and field this change is associated with to that of the change object before this one. That is, undo this action.
   def undo(author)
    @changes = Change.find(:all, :conditions => ["object_path = ? and date < ? and data_changed = ?", self.object_path, self.date, self.data_changed])
    if @changes[@changes.length-2]
    @changes[@changes.length-2].revert(author)  #Change the field back to how it was last change.
    @changes_to_delete = Change.find(:all, :conditions => ["object_path = ? and date >= ? and data_changed = ?", self.object_path, self.date, self.data_changed]) 
    @changes_to_delete.each { |c| c.destroy }  #Delete all changes after and including this one
    end
   end
end