# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

#Models the users table, which stores information about users with permission to edit content.
#Has many changes.
#Email must be unique
#Email, password, user_level, first name, and last name must all not be null.
#Passwords must match their confirmations on creation.
#User level must be an integer between 1 and 3.
class User < ActiveRecord::Base
	has_many :changes
	
	attr_accessor :password, :password_confirmation, :old_password
	attr_accessible :email, :password, :user_level, :password_confirmation, :old_password
	validates_uniqueness_of :email
	validates_presence_of :email, :password, :user_level, :first_name, :last_name, :on => :create
	validates_confirmation_of :password, :on => :create
	validates_numericality_of :user_level
	validates_inclusion_of :user_level, :in=>1..3, :message => "must be either 1, 2, or 3"
  validates_format_of :email, :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/, :on => :create,
		:message => "must be a valid e-mail address."
#Before a user's creation, hash the entered password and store the result as hashed_password.
	def before_create
		self.hashed_password = User.hash_password(self.password)
	end

#Before a user is updated, hash the new password as in before_create.
	def before_update
		before_create
	end
	
#Attempts to login a user. If a user with matching email and password is found, returns the user.
#Returns null otherwise. Not for use outside of this model.
	def self.login(email, password)
	  hashed_password = hash_password(password || "")
	  find(:first, :conditions => ["email = ? and hashed_password = ?", email, hashed_password])
	end
	
#Attempts to login a user. If a user with matching email and password is found, returns the user.
#Returns null otherwise. Intended for use outside of this model.
	def try_to_login
	  User.login(self.email, self.password)
        end

#Hashes the given password using the Digest/SHA1 library function and returns the result.
	private
	def self.hash_password(password)
		Digest::SHA1.hexdigest(password)
	end
	

end

